﻿using Breeze.AspNetCore;
using Breeze.Persistence;
using Breeze.Persistence.EFCore;
using Microsoft.AspNetCore.Cors;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using owl.owltime.data;
using owl.owltime.data.entities;
using owl.owltime.data.legacy;
using System;
using System.Data.SqlClient;
using System.Linq;

namespace owl.owltime.api.Controllers
{
    [Route("breeze/[controller]/[action]")]
    [BreezeQueryFilter]
    //[EnableCors()]
    public class PoController : Controller
    {
        IHostingEnvironment _env;
        PoContext _poContext;
        OwlTimeContext _otContext;
        EFPersistenceManager<PoContext> _persistenceManager = new EFPersistenceManager<PoContext>();

        public PoController(PoContext poContext, OwlTimeContext otContext, IHostingEnvironment env)
        {
            _poContext = poContext;
            _otContext = otContext;
            _persistenceManager = new EFPersistenceManager<PoContext>(_poContext);
            _persistenceManager.BeforeSaveEntityDelegate += beforeSaveEntity;
            this._env = env;

            JsonSerializerSettings sets = new JsonSerializerSettings
            {
                PreserveReferencesHandling = PreserveReferencesHandling.Objects
            };
        }

        private bool beforeSaveEntity(EntityInfo entityInfo)
        {
            if (entityInfo.Entity.GetType() == typeof(Shipment) && (entityInfo.EntityState == Breeze.Persistence.EntityState.Added || entityInfo.EntityState == Breeze.Persistence.EntityState.Modified))
            {
                var shipment = entityInfo.Entity as Shipment;
                shipment.UserName = User.Identity.Name;
            }

            if (entityInfo.Entity.GetType() == typeof(PurchaseOrder) && (entityInfo.EntityState == Breeze.Persistence.EntityState.Added))
            {
                var po = entityInfo.Entity as PurchaseOrder;
                var job = _otContext.Jobs.FirstOrDefault(j => j.PartNumber == po.Material);

                if (job == null)
                {
                    throw new Exception("Part does not exist in OWLTIME");
                }
            }

            return true;
        }

        [HttpGet]
        public IActionResult Metadata()
        {
            return Ok(_persistenceManager.Metadata());
        }

        [HttpPost]
        public SaveResult SaveChanges([FromBody] JObject saveBundle)
        {
            return _persistenceManager.SaveChanges(saveBundle);
        }

        [HttpGet]
        public IQueryable<VendorReport> VendorReports()
        {
            return _persistenceManager.Context.VendorReports;
        }

        [HttpGet]
        public IQueryable<PurchaseOrder> PurchaseOrders()
        {
            return _persistenceManager.Context.PurchaseOrders;
        }

        [HttpGet]
        public IQueryable<PurchaseOrder> PurchaseOrdersForUser()
        {
            var owlTimeContext = new OwlTimeContext();

            var user = this.User.Identity.Name;
            if (_env.IsDevelopment())
            {
                if (string.IsNullOrEmpty(user)) user =  @"OWLINC\timeadmin";
            }

            SqlParameter userParam = new SqlParameter("@user", user);
            return _persistenceManager.Context.PurchaseOrders.FromSql("GetPurchaseOrdersForUserCore @user", userParam).AsQueryable();
        }

        [HttpGet]
        public IQueryable<Shipment> Shipments()
        {
            return _persistenceManager.Context.Shipments;
        }

        [HttpGet]
        public IQueryable<LogEntry> LogEntries()
        {
            return _persistenceManager.Context.LogEntries;
        }

        [HttpGet]
        public IQueryable<PoStatus> PoStatuses()
        {
            return _persistenceManager.Context.PoStatuses;
        }

        [HttpGet]
        public IQueryable<Document> Documents()
        {
            return _persistenceManager.Context.Documents; //.Select(d => new Document { DocumentId = d.DocumentId, DocumentName = d.DocumentName, PurchaseOrderId = d.PurchaseOrderId, CreatedDate = d.CreatedDate, ModifiedDate = d.ModifiedDate });
        }
    }
}
