import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from "@angular/router";
import { Moment } from 'moment';
import { GridComponent, SelectionService } from '@syncfusion/ej2-angular-grids';

import { PurchaseOrder, PoStatus } from '../../model/entity-model';
import { PoRepositoryService } from '../../services/po-repository.service';

@Component({
  selector: 'app-po-search',
  templateUrl: './po-search.component.html',
  styleUrls: ['./po-search.component.css'],
  providers: [SelectionService]
})

export class PoSearchComponent implements OnInit {
  private searchPoNum: string;
  private searchPartNum: string;
  private searchDescription: string;
  private poStatus: PoStatus;
  private searchReceivedStart: Moment;
  private searchReceivedEnd: Moment;
  private searchShipped: Moment;
  public pageSettings = { pageSizes: true, pageSize: 10, enableQueryString: true };
  public selectionSettings = { type: 'Single' };

  @ViewChild('grid')
  public grid: GridComponent;

  //private toolbarSettings = { showToolbar: true, toolbarItems: [ej.Grid.ToolBarItems.WordExport, ej.Grid.ToolBarItems.PdfExport] }

  constructor(private _poRepository: PoRepositoryService, private route: ActivatedRoute, private router: Router) { }

  public purchaseOrders: PurchaseOrder[] = [];
  public statuses: PoStatus[] = [];


  ngOnInit() {
    this.getStatuses();  
    this.poStatus = <PoStatus> { poStatusId: 1, description: 'Active' };  
    this.search(null);
  }

  search(event) {
    this._poRepository.searchPurchaseOrders(
      this.searchPoNum,
      this.searchPartNum,
      this.searchDescription,
      this.poStatus.poStatusId,
      this.searchReceivedStart,
      this.searchReceivedEnd,
      this.searchShipped
    ).then(
      purchaseOrders => {
        this.purchaseOrders = <any>purchaseOrders; // success
      },
      error => console.error(error) // error
    );
  }

  getStatuses() {
    this._poRepository.getStatuses().then(
      s => this.statuses = <any>s, // success
      error => console.error(error) // error
    );
  }

  rowSelected(args: any) {
    let id: number = args.data.purchaseOrderId;

    if (id && id > 0) {
      this.router.navigate(['app-po-details'], { queryParams: { id: id }});
    }
  }
}
